/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package hypertree;

import java.awt.event.*;


/**
 * The HTAction class manage the action on the hypertree :
 * drag of a node...
 *
 * @author Christophe Bouthier [bouthier@loria.fr]
 * @version 1.0
 */
class HTAction
    extends MouseAdapter
    implements MouseMotionListener {

    private HTDraw   model      = null; // the drawing model
    private HTView   view       = null; // the view

    private HTCoordE startPoint = null; // starting point of dragging
    private HTCoordE endPoint   = null; // ending point of dragging
    private HTCoordS clickPoint = null; // clicked point

    private boolean  kleinMode = false;


  /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param model    the drawing model
     * @param view     the view
     */
    HTAction(HTDraw model, HTView view) {
        this.model = model;
        this.view = view;
        startPoint = new HTCoordE();
        endPoint = new HTCoordE();
        clickPoint = new HTCoordS();
    }


  /* --- MouseAdapter --- */

    /**
     * Called when a user pressed the mouse button
     * on the hyperbolic tree.
     * Used to get the starting point of the drag.
     *
     * @param e    the MouseEvent generated when clicking
     */
    public void mousePressed(MouseEvent e) {
//Added by Marv---->
        if (view.getFastMode()) {
            model.fastMode(true);
        }
//<----Added by Marv
        if (e.isShiftDown()) {
            model.fastMode(true);
        }
        if (e.isControlDown()) {
            model.longNameMode(true);
        }
        startPoint.projectionStoE(e.getX(), e.getY(),
                                  model.getSOrigin(),
                                  model.getSMax());
    }

    /**
     * Called when a user release the mouse button
     * on the hyperbolic tree.
     * Used to signal the end of the translation.
     *
     * @param e    not used here
     */
    public void mouseReleased(MouseEvent e) {
        if (view.getFastMode() == false) {
            model.fastMode(false);
        }
        if (view.getLongNameMode() == false) {
            model.longNameMode(false);
        }
        model.endTranslation();
    }

    /**
     * Called when a user clicked on the hyperbolic tree.
     * Used to put the corresponding node (if any) at the
     * center of the hyperbolic tree.
     *
     * @param e    the MouseEvent generated when clicking
     */
    public void mouseClicked(MouseEvent e) {
        if (e.isShiftDown()) {
            model.restore();
        } else if (e.isControlDown()) {
            switchKleinMode();
        } else {
            clickPoint.x = e.getX();
            clickPoint.y = e.getY();
            HTDrawNode node = model.findNode(clickPoint);
            if (node != null) {
                model.translateToOrigin(node);
            }
        }
    }

    private void switchKleinMode() {
        if (kleinMode) {
            kleinMode = false;
        } else {
            kleinMode = true;
        }
        model.kleinMode(kleinMode);
    }

  /* --- MouseMotionListener --- */

    /**
     * Called when a used drag the mouse on the hyperbolic tree.
     * Used to translate the hypertree, thus moving the focus.
     *
     * @param e    the MouseEvent generated when draging
     */
    public void mouseDragged(MouseEvent e) {
        if (startPoint.isValid()) {
            endPoint.projectionStoE(e.getX(), e.getY(),
                                    model.getSOrigin(),
                                    model.getSMax());
            if (endPoint.isValid()) {
                model.translate(startPoint, endPoint);
            }
        }
    }

    /**
     * Called when the mouse mouve into the hyperbolic tree.
     * Not used here.
     *
     * @param e    the MouseEvent generated when mouving
     */
    public void mouseMoved(MouseEvent e) {}

}

