/*
 * HTDraw.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001-2003 Christophe Bouthier
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package hypertree;

import java.awt.Graphics;
import java.awt.Color;
import java.awt.Insets;
//PDA
import javax.swing.SwingUtilities;
//PDA
//Added by Marv---->
import java.util.Vector;
//<----Added by Marv


/**
 * The HTDraw class implements the drawing model for the HTView.
 *
 * @author Christophe Bouthier [bouthier@loria.fr]
 * @version 1.0
 */
class HTDraw {

    private final static int NBR_FRAMES = 5; // number of intermediates
                                             // animation frames

    private HTModel    model    = null;  // the tree model
    private HTView     view     = null;  // the view using this drawing model
    private HTDrawNode drawRoot = null;  // the root of the drawing tree

    private HTCoordS   sOrigin  = null;  // origin of the screen plane
    private HTCoordS   sMax     = null;  // max point in the screen plane

    private int        border   = 5;

    private double[]   ray      = null;

    private boolean    fastMode = false; // fast mode
    private boolean    longNameMode = false; // long name mode
    private boolean    transNotCorrected = false;
    private boolean    kleinMode = false; // klein mode
// NEU
    private double     projType = 0.0; // projection type
// FIN NEU

    public static boolean isTranslating = false;


  /* --- Constructor --- */

    /**
     * Constructor.
     *
     * @param model    the tree model to draw
     * @param view     the view using this drawing model
     */
    HTDraw(HTModel model, HTView view) {
        this.view = view;
        this.model = model;
        HTModelNode root = model.getRoot();
        sOrigin = new HTCoordS();
        sMax = new HTCoordS();

        ray = new double[4];
        ray[0] = model.getLength();

        for (int i = 1; i < ray.length; i++) {
            ray[i] = (ray[0] + ray[i - 1]) / (1 + (ray[0] * ray[i - 1]));
        }

        if (root.isLeaf()) {
            drawRoot = new HTDrawNode(null, root, this);
        } else {
            drawRoot = new HTDrawNodeComposite(null,
                                            (HTModelNodeComposite) root, this);
        }
    }


   /* --- border --- */

    /**
     * Returns the border.
     *
     * @return    the border
     */
    int getBorder() {
        return border;
    }


  /* --- Screen coordinates --- */

    /**
     * Refresh the screen coordinates of the drawing tree.
     */
    void refreshScreenCoordinates() {
        Insets insets = view.getInsets();
        //PDA
        sMax.x = (view.getWidth() - insets.left - insets.right) / 2 - border;
        sMax.y = (view.getHeight() - insets.top - insets.bottom) / 2 - border;
        //sMax.x = (view.getSize().width - insets.left - insets.right) / 2;
        //sMax.y = (view.getSize().height - insets.top - insets.bottom) / 2;
        //PDA
        sOrigin.x = sMax.x + insets.left + border;
        sOrigin.y = sMax.y + insets.top + border;
        drawRoot.refreshScreenCoordinates(sOrigin, sMax);
    }

    /**
     * Returns the origin of the screen plane.
     * WARNING : this is not a copy but the original object.
     *
     * @return    the origin of the screen plane
     */
    HTCoordS getSOrigin() {
        return sOrigin;
    }

    /**
     * Return the point representing the up right corner
     * of the screen plane, thus giving x and y maxima.
     * WARNING : this is not a copy but the original object.
     *
     * @return    the max point
     */
    HTCoordS getSMax() {
        return sMax;
    }


  /* --- Drawing --- */

    /**
     * Draws the branches of the hyperbolic tree.
     *
     * @param g    the graphic context
     */
    void drawBranches(Graphics g) {
        drawRoot.drawBranches(g);
    }

    /**
     * Draws the nodes of the hyperbolic tree.
     *
     * @param g    the graphic context
     */
    void drawNodes(Graphics g) {
        drawRoot.drawNodes(g);
    }


  /* --- Translation --- */

    /**
     * Translates the hyperbolic tree by the given vector.
     *
     * param t    the translation vector
     */
    void translate(HTCoordE zs, HTCoordE ze) {
        isTranslating = true;

        HTCoordE zst = null;
        HTCoordE zet = null;

        if (kleinMode) {
            zst = zs.kToP();
            zet = ze.kToP();
        } else if (projType != 0.0) {
            zst = zs.zToP(projType);
            zet = ze.zToP(projType);
        } else {
            zst = new HTCoordE(zs);
            zet = new HTCoordE(ze);
        }

        if (transNotCorrected) {
            HTCoordE v = new HTCoordE();
            double de = zet.d2();
            double ds = zst.d2();
            double dd = 1.0 - de * ds;
            v.x = (zet.x * ( 1.0 - ds) - zst.x * (1.0 - de)) / dd;
            v.y = (zet.y * ( 1.0 - ds) - zst.y * (1.0 - de)) / dd;
            if (v.isValid()) {
                 drawRoot.translate(v);
            }
        } else {
            HTCoordE zo = new HTCoordE(drawRoot.getOldCoordinates());
            zo.x = - zo.x;
            zo.y = - zo.y;
            HTCoordE zs2 = new HTCoordE(zst);

            zs2.translate(zo);

            HTCoordE t = new HTCoordE();
            double de = zet.d2();
            double ds = zs2.d2();
            double dd = 1.0 - de * ds;
            t.x = (zet.x * ( 1.0 - ds) - zs2.x * (1.0 - de)) / dd;
            t.y = (zet.y * ( 1.0 - ds) - zs2.y * (1.0 - de)) / dd;

            if (t.isValid()) {

                // alpha = 1 + conj(zo)*t
                HTCoordE alpha = new HTCoordE();
                alpha.x = 1 + (zo.x * t.x) + (zo.y * t.y);
                alpha.y = (zo.x * t.y) - (zo.y * t.x);
                // beta = zo + t
                HTCoordE beta = new HTCoordE();
                beta.x = zo.x + t.x;
                beta.y = zo.y + t.y;

                drawRoot.specialTrans(alpha, beta);
             }
        }
        view.repaint();
    }

    void translateCorrected(HTCoordE zs, HTCoordE ze) {
        isTranslating = true;

        HTCoordE zo = new HTCoordE(drawRoot.getOldCoordinates());
        zo.x = - zo.x;
        zo.y = - zo.y;
        HTCoordE zs2 = new HTCoordE(zs);

        zs2.translate(zo);

        HTCoordE t = new HTCoordE();
        double de = ze.d2();
        double ds = zs2.d2();
        double dd = 1.0 - de * ds;
        t.x = (ze.x * ( 1.0 - ds) - zs2.x * (1.0 - de)) / dd;
        t.y = (ze.y * ( 1.0 - ds) - zs2.y * (1.0 - de)) / dd;

        if (t.isValid()) {

            // alpha = 1 + conj(zo)*t
            HTCoordE alpha = new HTCoordE();
            alpha.x = 1 + (zo.x * t.x) + (zo.y * t.y);
            alpha.y = (zo.x * t.y) - (zo.y * t.x);
            // beta = zo + t
            HTCoordE beta = new HTCoordE();
            beta.x = zo.x + t.x;
            beta.y = zo.y + t.y;

            drawRoot.specialTrans(alpha, beta);
            view.repaint();
        }
    }

    /**
     * Signal that the translation ended.
     */
    void endTranslation() {
        isTranslating = false;
        drawRoot.endTranslation();
//Added by Marv---->
        if (view.getFastMode()) {
            fastMode(false);
        }
//<----Added by Marv
        view.repaint();
    }

    /**
     * Translate the hyperbolic tree so that the given node
     * is put at the origin of the hyperbolic tree.
     *
     * @param node    the given HTDrawNode
     */
    void translateToOrigin(HTDrawNode node) {
//Added by Marv---->
        if (view.getFastMode()) {
            fastMode(true);
        }
//<----Added by Marv
        view.stopMouseListening();
        isTranslating = true;
        AnimThread t = new AnimThread(node);
        t.start();
    }

    /**
     * Restores the hyperbolic tree to its origin.
     */
    void restore() {
        drawRoot.restore();
        view.repaint();
    }

    /**
     * Sets the fast mode, where nodes are no more drawed.
     *
     * @param mode    setting on or off.
     */
    void fastMode(boolean mode) {
        if (mode != fastMode) {
            fastMode = mode;
            drawRoot.fastMode(mode);
            if (mode == false) {
                view.repaint();
            }
        }
    }

    boolean getFastMode() {
        return fastMode;
    }

    /**
     * Sets the long name mode, where full names are drawn.
     *
     * @param mode    setting on or off.
     */
    void longNameMode(boolean mode) {
        if (mode != longNameMode) {
            longNameMode = mode;
            drawRoot.longNameMode(mode);
            view.repaint();
        }
    }

    boolean getLongNameMode() {
        return longNameMode;
    }

    void transNotCorrectedMode(boolean mode) {
        transNotCorrected = mode;
    }

    boolean getTransNotCorrectedMode() {
        return transNotCorrected;
    }

    void setQuadMode(boolean mode) {
        drawRoot.setQuadMode(mode);
    }

    /**
     * Sets the klein mode.
     *
     * @param mode    setting on or off.
     */
    void kleinMode(boolean mode) {
        if (mode != kleinMode) {
            HTCoordE zo = new HTCoordE(drawRoot.getCoordinates());

            kleinMode = mode;
            if (kleinMode) {
                model.setLengthKlein();
            } else {
                model.setLengthPoincare();
            }
            model.layoutHyperbolicTree();
            drawRoot.kleinMode(mode);

            restore();
            if (! kleinMode) { // poincare
                translateCorrected(new HTCoordE(), zo.pToK());
            } else {           // klein
                translateCorrected(new HTCoordE(), zo.kToP());
            }
            endTranslation();
        }
    }

// NEU
    /**
     * Change projection type.
     *
     * @param z    the new value of proj type
     */
    void changeProjType(double z) {
        if (z != projType) {
            HTCoordE zo = new HTCoordE(drawRoot.getCoordinates());

            double oldProjType = projType;
            projType = z;
            //model.layoutHyperbolicTree();
            drawRoot.changeProjType(z);


            restore();
            translateCorrected(new HTCoordE(), zo.pToZ(oldProjType).zToP(z));
            endTranslation();
        }

    }

    double getProjType() {
        return projType;
    }
// FIN NEU


  /* --- Node searching --- */

    /**
     * Returns the node (if any) whose screen coordinates' zone
     * contains thoses given in parameters.
     *
     * @param zs    the given screen coordinate
     * @return      the searched HTDrawNode if found;
     *              <CODE>null</CODE> otherwise
     */
    HTDrawNode findNode(HTCoordS zs) {
        return drawRoot.findNode(zs);
    }
//Added by Marv---->
    HTDrawNode findNodebyName(String pName, Vector pAlreadyFoundNodes) {
        return drawRoot.findNodebyName(pName, pAlreadyFoundNodes);
    }
//<----Added by Marv

  /* --- Inner animation thread --- */

    /**
     * The AnimThread class implements the thread that do the animation
     * when clicking on a node.
     */
    class AnimThread
        extends Thread {

        private HTDrawNode node  = null; // node to put at the origin
        private Runnable   tTask = null; // translation task

        /**
         * Constructor.
         *
         * @param node    the node to put at the origin
         */
        AnimThread(HTDrawNode node) {
            this.node = node;
        }

        /**
         * Do the animation.
         */
        public void run() {
            HTCoordE zn = node.getOldCoordinates();
            HTCoordE zf = new HTCoordE();

            int frames = NBR_FRAMES;
            int nodes = model.getNumberOfNodes();

            double d = zn.d();
            for (int i = 0; i < ray.length; i++) {
            	if (d > ray[i]) {
                	frames += NBR_FRAMES / 2;
            	}
            }

            double factorX = zn.x / frames;
            double factorY = zn.y / frames;

            for (int i = 1; i < frames; i++) {
                zf.x = zn.x - (i * factorX);
                zf.y = zn.y - (i * factorY);
                tTask = new TranslateThread(zn, zf);
                //PDA
                try {
                    SwingUtilities.invokeAndWait(tTask);
                } catch (Exception e) {
                    e.printStackTrace();
                }
                //tTask.run();
                //PDA
            }

            zf.x = 0.0;
            zf.y = 0.0;
            tTask = new LastTranslateThread(zn, zf);
            //PDA
            try {
                SwingUtilities.invokeAndWait(tTask);
            } catch (Exception e) {
                e.printStackTrace();
            }
            //tTask.run();
            //PDA
            isTranslating = false;
        }

      /* --- Inner's inner --- */

        class TranslateThread
            implements Runnable {

            HTCoordE zStart = null;
            HTCoordE zEnd   = null;

            TranslateThread(HTCoordE z1, HTCoordE z2) {
                zStart = z1;
                zEnd = z2;
            }

            public void run() {
                translate(zStart, zEnd);
			    view.repaint();
            }
        }

        class LastTranslateThread
            implements Runnable {

            HTCoordE zStart = null;
            HTCoordE zEnd   = null;

            LastTranslateThread(HTCoordE z1, HTCoordE z2) {
                zStart = z1;
                zEnd = z2;
            }

            public void run() {
                translate(zStart, zEnd);
                endTranslation();
			    view.repaint();
			    view.startMouseListening();
            }
        }


    }

}

